# coding: utf-8
# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20221001


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class InputLocation(object):
    """
    document location and other meta data about documents
    For TXT only ObjectStoragePrefixLocation supported
    For CSV only ObjectStorageFileNameLocation is supported
    For this release only one file is supported for ObjectStorageFileNameLocation i.e CSV file type
    """

    #: A constant which can be used with the location_type property of a InputLocation.
    #: This constant has a value of "OBJECT_STORAGE_PREFIX"
    LOCATION_TYPE_OBJECT_STORAGE_PREFIX = "OBJECT_STORAGE_PREFIX"

    #: A constant which can be used with the location_type property of a InputLocation.
    #: This constant has a value of "OBJECT_STORAGE_FILE_LIST"
    LOCATION_TYPE_OBJECT_STORAGE_FILE_LIST = "OBJECT_STORAGE_FILE_LIST"

    def __init__(self, **kwargs):
        """
        Initializes a new InputLocation object with values from keyword arguments. This class has the following subclasses and if you are using this class as input
        to a service operations then you should favor using a subclass over the base class:

        * :class:`~oci.ai_language.models.ObjectStoragePrefixLocation`
        * :class:`~oci.ai_language.models.ObjectStorageFileNameLocation`

        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param location_type:
            The value to assign to the location_type property of this InputLocation.
            Allowed values for this property are: "OBJECT_STORAGE_PREFIX", "OBJECT_STORAGE_FILE_LIST", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type location_type: str

        """
        self.swagger_types = {
            'location_type': 'str'
        }

        self.attribute_map = {
            'location_type': 'locationType'
        }

        self._location_type = None

    @staticmethod
    def get_subtype(object_dictionary):
        """
        Given the hash representation of a subtype of this class,
        use the info in the hash to return the class of the subtype.
        """
        type = object_dictionary['locationType']

        if type == 'OBJECT_STORAGE_PREFIX':
            return 'ObjectStoragePrefixLocation'

        if type == 'OBJECT_STORAGE_FILE_LIST':
            return 'ObjectStorageFileNameLocation'
        else:
            return 'InputLocation'

    @property
    def location_type(self):
        """
        **[Required]** Gets the location_type of this InputLocation.
        locationType

        Allowed values for this property are: "OBJECT_STORAGE_PREFIX", "OBJECT_STORAGE_FILE_LIST", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The location_type of this InputLocation.
        :rtype: str
        """
        return self._location_type

    @location_type.setter
    def location_type(self, location_type):
        """
        Sets the location_type of this InputLocation.
        locationType


        :param location_type: The location_type of this InputLocation.
        :type: str
        """
        allowed_values = ["OBJECT_STORAGE_PREFIX", "OBJECT_STORAGE_FILE_LIST"]
        if not value_allowed_none_or_none_sentinel(location_type, allowed_values):
            location_type = 'UNKNOWN_ENUM_VALUE'
        self._location_type = location_type

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other
