# coding: utf-8
# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200407


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class CreatePipelineDetails(object):
    """
    Details with which to create a pipeline.
    """

    #: A constant which can be used with the recipe_type property of a CreatePipelineDetails.
    #: This constant has a value of "ZERO_ETL"
    RECIPE_TYPE_ZERO_ETL = "ZERO_ETL"

    #: A constant which can be used with the license_model property of a CreatePipelineDetails.
    #: This constant has a value of "LICENSE_INCLUDED"
    LICENSE_MODEL_LICENSE_INCLUDED = "LICENSE_INCLUDED"

    #: A constant which can be used with the license_model property of a CreatePipelineDetails.
    #: This constant has a value of "BRING_YOUR_OWN_LICENSE"
    LICENSE_MODEL_BRING_YOUR_OWN_LICENSE = "BRING_YOUR_OWN_LICENSE"

    def __init__(self, **kwargs):
        """
        Initializes a new CreatePipelineDetails object with values from keyword arguments. This class has the following subclasses and if you are using this class as input
        to a service operations then you should favor using a subclass over the base class:

        * :class:`~oci.golden_gate.models.CreateZeroEtlPipelineDetails`

        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param recipe_type:
            The value to assign to the recipe_type property of this CreatePipelineDetails.
            Allowed values for this property are: "ZERO_ETL"
        :type recipe_type: str

        :param display_name:
            The value to assign to the display_name property of this CreatePipelineDetails.
        :type display_name: str

        :param description:
            The value to assign to the description property of this CreatePipelineDetails.
        :type description: str

        :param compartment_id:
            The value to assign to the compartment_id property of this CreatePipelineDetails.
        :type compartment_id: str

        :param license_model:
            The value to assign to the license_model property of this CreatePipelineDetails.
            Allowed values for this property are: "LICENSE_INCLUDED", "BRING_YOUR_OWN_LICENSE"
        :type license_model: str

        :param freeform_tags:
            The value to assign to the freeform_tags property of this CreatePipelineDetails.
        :type freeform_tags: dict(str, str)

        :param defined_tags:
            The value to assign to the defined_tags property of this CreatePipelineDetails.
        :type defined_tags: dict(str, dict(str, object))

        :param locks:
            The value to assign to the locks property of this CreatePipelineDetails.
        :type locks: list[oci.golden_gate.models.ResourceLock]

        :param source_connection_details:
            The value to assign to the source_connection_details property of this CreatePipelineDetails.
        :type source_connection_details: oci.golden_gate.models.SourcePipelineConnectionDetails

        :param target_connection_details:
            The value to assign to the target_connection_details property of this CreatePipelineDetails.
        :type target_connection_details: oci.golden_gate.models.TargetPipelineConnectionDetails

        """
        self.swagger_types = {
            'recipe_type': 'str',
            'display_name': 'str',
            'description': 'str',
            'compartment_id': 'str',
            'license_model': 'str',
            'freeform_tags': 'dict(str, str)',
            'defined_tags': 'dict(str, dict(str, object))',
            'locks': 'list[ResourceLock]',
            'source_connection_details': 'SourcePipelineConnectionDetails',
            'target_connection_details': 'TargetPipelineConnectionDetails'
        }

        self.attribute_map = {
            'recipe_type': 'recipeType',
            'display_name': 'displayName',
            'description': 'description',
            'compartment_id': 'compartmentId',
            'license_model': 'licenseModel',
            'freeform_tags': 'freeformTags',
            'defined_tags': 'definedTags',
            'locks': 'locks',
            'source_connection_details': 'sourceConnectionDetails',
            'target_connection_details': 'targetConnectionDetails'
        }

        self._recipe_type = None
        self._display_name = None
        self._description = None
        self._compartment_id = None
        self._license_model = None
        self._freeform_tags = None
        self._defined_tags = None
        self._locks = None
        self._source_connection_details = None
        self._target_connection_details = None

    @staticmethod
    def get_subtype(object_dictionary):
        """
        Given the hash representation of a subtype of this class,
        use the info in the hash to return the class of the subtype.
        """
        type = object_dictionary['recipeType']

        if type == 'ZERO_ETL':
            return 'CreateZeroEtlPipelineDetails'
        else:
            return 'CreatePipelineDetails'

    @property
    def recipe_type(self):
        """
        **[Required]** Gets the recipe_type of this CreatePipelineDetails.
        The type of the recipe

        Allowed values for this property are: "ZERO_ETL"


        :return: The recipe_type of this CreatePipelineDetails.
        :rtype: str
        """
        return self._recipe_type

    @recipe_type.setter
    def recipe_type(self, recipe_type):
        """
        Sets the recipe_type of this CreatePipelineDetails.
        The type of the recipe


        :param recipe_type: The recipe_type of this CreatePipelineDetails.
        :type: str
        """
        allowed_values = ["ZERO_ETL"]
        if not value_allowed_none_or_none_sentinel(recipe_type, allowed_values):
            raise ValueError(
                f"Invalid value for `recipe_type`, must be None or one of {allowed_values}"
            )
        self._recipe_type = recipe_type

    @property
    def display_name(self):
        """
        **[Required]** Gets the display_name of this CreatePipelineDetails.
        An object's Display Name.


        :return: The display_name of this CreatePipelineDetails.
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name):
        """
        Sets the display_name of this CreatePipelineDetails.
        An object's Display Name.


        :param display_name: The display_name of this CreatePipelineDetails.
        :type: str
        """
        self._display_name = display_name

    @property
    def description(self):
        """
        Gets the description of this CreatePipelineDetails.
        Metadata about this specific object.


        :return: The description of this CreatePipelineDetails.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this CreatePipelineDetails.
        Metadata about this specific object.


        :param description: The description of this CreatePipelineDetails.
        :type: str
        """
        self._description = description

    @property
    def compartment_id(self):
        """
        **[Required]** Gets the compartment_id of this CreatePipelineDetails.
        The `OCID`__ of the compartment being referenced.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The compartment_id of this CreatePipelineDetails.
        :rtype: str
        """
        return self._compartment_id

    @compartment_id.setter
    def compartment_id(self, compartment_id):
        """
        Sets the compartment_id of this CreatePipelineDetails.
        The `OCID`__ of the compartment being referenced.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param compartment_id: The compartment_id of this CreatePipelineDetails.
        :type: str
        """
        self._compartment_id = compartment_id

    @property
    def license_model(self):
        """
        **[Required]** Gets the license_model of this CreatePipelineDetails.
        The Oracle license model that applies to a Deployment.

        Allowed values for this property are: "LICENSE_INCLUDED", "BRING_YOUR_OWN_LICENSE"


        :return: The license_model of this CreatePipelineDetails.
        :rtype: str
        """
        return self._license_model

    @license_model.setter
    def license_model(self, license_model):
        """
        Sets the license_model of this CreatePipelineDetails.
        The Oracle license model that applies to a Deployment.


        :param license_model: The license_model of this CreatePipelineDetails.
        :type: str
        """
        allowed_values = ["LICENSE_INCLUDED", "BRING_YOUR_OWN_LICENSE"]
        if not value_allowed_none_or_none_sentinel(license_model, allowed_values):
            raise ValueError(
                f"Invalid value for `license_model`, must be None or one of {allowed_values}"
            )
        self._license_model = license_model

    @property
    def freeform_tags(self):
        """
        Gets the freeform_tags of this CreatePipelineDetails.
        A simple key-value pair that is applied without any predefined name, type, or scope. Exists
        for cross-compatibility only.

        Example: `{\"bar-key\": \"value\"}`


        :return: The freeform_tags of this CreatePipelineDetails.
        :rtype: dict(str, str)
        """
        return self._freeform_tags

    @freeform_tags.setter
    def freeform_tags(self, freeform_tags):
        """
        Sets the freeform_tags of this CreatePipelineDetails.
        A simple key-value pair that is applied without any predefined name, type, or scope. Exists
        for cross-compatibility only.

        Example: `{\"bar-key\": \"value\"}`


        :param freeform_tags: The freeform_tags of this CreatePipelineDetails.
        :type: dict(str, str)
        """
        self._freeform_tags = freeform_tags

    @property
    def defined_tags(self):
        """
        Gets the defined_tags of this CreatePipelineDetails.
        Tags defined for this resource. Each key is predefined and scoped to a namespace.

        Example: `{\"foo-namespace\": {\"bar-key\": \"value\"}}`


        :return: The defined_tags of this CreatePipelineDetails.
        :rtype: dict(str, dict(str, object))
        """
        return self._defined_tags

    @defined_tags.setter
    def defined_tags(self, defined_tags):
        """
        Sets the defined_tags of this CreatePipelineDetails.
        Tags defined for this resource. Each key is predefined and scoped to a namespace.

        Example: `{\"foo-namespace\": {\"bar-key\": \"value\"}}`


        :param defined_tags: The defined_tags of this CreatePipelineDetails.
        :type: dict(str, dict(str, object))
        """
        self._defined_tags = defined_tags

    @property
    def locks(self):
        """
        Gets the locks of this CreatePipelineDetails.
        Locks associated with this resource.


        :return: The locks of this CreatePipelineDetails.
        :rtype: list[oci.golden_gate.models.ResourceLock]
        """
        return self._locks

    @locks.setter
    def locks(self, locks):
        """
        Sets the locks of this CreatePipelineDetails.
        Locks associated with this resource.


        :param locks: The locks of this CreatePipelineDetails.
        :type: list[oci.golden_gate.models.ResourceLock]
        """
        self._locks = locks

    @property
    def source_connection_details(self):
        """
        **[Required]** Gets the source_connection_details of this CreatePipelineDetails.

        :return: The source_connection_details of this CreatePipelineDetails.
        :rtype: oci.golden_gate.models.SourcePipelineConnectionDetails
        """
        return self._source_connection_details

    @source_connection_details.setter
    def source_connection_details(self, source_connection_details):
        """
        Sets the source_connection_details of this CreatePipelineDetails.

        :param source_connection_details: The source_connection_details of this CreatePipelineDetails.
        :type: oci.golden_gate.models.SourcePipelineConnectionDetails
        """
        self._source_connection_details = source_connection_details

    @property
    def target_connection_details(self):
        """
        **[Required]** Gets the target_connection_details of this CreatePipelineDetails.

        :return: The target_connection_details of this CreatePipelineDetails.
        :rtype: oci.golden_gate.models.TargetPipelineConnectionDetails
        """
        return self._target_connection_details

    @target_connection_details.setter
    def target_connection_details(self, target_connection_details):
        """
        Sets the target_connection_details of this CreatePipelineDetails.

        :param target_connection_details: The target_connection_details of this CreatePipelineDetails.
        :type: oci.golden_gate.models.TargetPipelineConnectionDetails
        """
        self._target_connection_details = target_connection_details

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other
