# coding: utf-8
# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200407

from .connection import Connection
from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class JavaMessageServiceConnection(Connection):
    """
    Represents the metadata of a Java Message Service Connection.
    """

    #: A constant which can be used with the technology_type property of a JavaMessageServiceConnection.
    #: This constant has a value of "ORACLE_WEBLOGIC_JMS"
    TECHNOLOGY_TYPE_ORACLE_WEBLOGIC_JMS = "ORACLE_WEBLOGIC_JMS"

    #: A constant which can be used with the security_protocol property of a JavaMessageServiceConnection.
    #: This constant has a value of "PLAIN"
    SECURITY_PROTOCOL_PLAIN = "PLAIN"

    #: A constant which can be used with the security_protocol property of a JavaMessageServiceConnection.
    #: This constant has a value of "TLS"
    SECURITY_PROTOCOL_TLS = "TLS"

    #: A constant which can be used with the security_protocol property of a JavaMessageServiceConnection.
    #: This constant has a value of "MTLS"
    SECURITY_PROTOCOL_MTLS = "MTLS"

    #: A constant which can be used with the authentication_type property of a JavaMessageServiceConnection.
    #: This constant has a value of "NONE"
    AUTHENTICATION_TYPE_NONE = "NONE"

    #: A constant which can be used with the authentication_type property of a JavaMessageServiceConnection.
    #: This constant has a value of "BASIC"
    AUTHENTICATION_TYPE_BASIC = "BASIC"

    def __init__(self, **kwargs):
        """
        Initializes a new JavaMessageServiceConnection object with values from keyword arguments. The default value of the :py:attr:`~oci.golden_gate.models.JavaMessageServiceConnection.connection_type` attribute
        of this class is ``JAVA_MESSAGE_SERVICE`` and it should not be changed.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param connection_type:
            The value to assign to the connection_type property of this JavaMessageServiceConnection.
            Allowed values for this property are: "GOLDENGATE", "KAFKA", "KAFKA_SCHEMA_REGISTRY", "MYSQL", "JAVA_MESSAGE_SERVICE", "MICROSOFT_SQLSERVER", "OCI_OBJECT_STORAGE", "ORACLE", "AZURE_DATA_LAKE_STORAGE", "POSTGRESQL", "AZURE_SYNAPSE_ANALYTICS", "SNOWFLAKE", "AMAZON_S3", "HDFS", "ORACLE_NOSQL", "MONGODB", "AMAZON_KINESIS", "AMAZON_REDSHIFT", "DB2", "REDIS", "ELASTICSEARCH", "GENERIC", "GOOGLE_CLOUD_STORAGE", "GOOGLE_BIGQUERY", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type connection_type: str

        :param id:
            The value to assign to the id property of this JavaMessageServiceConnection.
        :type id: str

        :param display_name:
            The value to assign to the display_name property of this JavaMessageServiceConnection.
        :type display_name: str

        :param description:
            The value to assign to the description property of this JavaMessageServiceConnection.
        :type description: str

        :param compartment_id:
            The value to assign to the compartment_id property of this JavaMessageServiceConnection.
        :type compartment_id: str

        :param freeform_tags:
            The value to assign to the freeform_tags property of this JavaMessageServiceConnection.
        :type freeform_tags: dict(str, str)

        :param defined_tags:
            The value to assign to the defined_tags property of this JavaMessageServiceConnection.
        :type defined_tags: dict(str, dict(str, object))

        :param system_tags:
            The value to assign to the system_tags property of this JavaMessageServiceConnection.
        :type system_tags: dict(str, dict(str, object))

        :param lifecycle_state:
            The value to assign to the lifecycle_state property of this JavaMessageServiceConnection.
            Allowed values for this property are: "CREATING", "UPDATING", "ACTIVE", "DELETING", "DELETED", "FAILED", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type lifecycle_state: str

        :param lifecycle_details:
            The value to assign to the lifecycle_details property of this JavaMessageServiceConnection.
        :type lifecycle_details: str

        :param time_created:
            The value to assign to the time_created property of this JavaMessageServiceConnection.
        :type time_created: datetime

        :param time_updated:
            The value to assign to the time_updated property of this JavaMessageServiceConnection.
        :type time_updated: datetime

        :param locks:
            The value to assign to the locks property of this JavaMessageServiceConnection.
        :type locks: list[oci.golden_gate.models.ResourceLock]

        :param vault_id:
            The value to assign to the vault_id property of this JavaMessageServiceConnection.
        :type vault_id: str

        :param key_id:
            The value to assign to the key_id property of this JavaMessageServiceConnection.
        :type key_id: str

        :param ingress_ips:
            The value to assign to the ingress_ips property of this JavaMessageServiceConnection.
        :type ingress_ips: list[oci.golden_gate.models.IngressIpDetails]

        :param nsg_ids:
            The value to assign to the nsg_ids property of this JavaMessageServiceConnection.
        :type nsg_ids: list[str]

        :param subnet_id:
            The value to assign to the subnet_id property of this JavaMessageServiceConnection.
        :type subnet_id: str

        :param routing_method:
            The value to assign to the routing_method property of this JavaMessageServiceConnection.
            Allowed values for this property are: "SHARED_SERVICE_ENDPOINT", "SHARED_DEPLOYMENT_ENDPOINT", "DEDICATED_ENDPOINT", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type routing_method: str

        :param does_use_secret_ids:
            The value to assign to the does_use_secret_ids property of this JavaMessageServiceConnection.
        :type does_use_secret_ids: bool

        :param technology_type:
            The value to assign to the technology_type property of this JavaMessageServiceConnection.
            Allowed values for this property are: "ORACLE_WEBLOGIC_JMS", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type technology_type: str

        :param should_use_jndi:
            The value to assign to the should_use_jndi property of this JavaMessageServiceConnection.
        :type should_use_jndi: bool

        :param jndi_connection_factory:
            The value to assign to the jndi_connection_factory property of this JavaMessageServiceConnection.
        :type jndi_connection_factory: str

        :param jndi_provider_url:
            The value to assign to the jndi_provider_url property of this JavaMessageServiceConnection.
        :type jndi_provider_url: str

        :param jndi_initial_context_factory:
            The value to assign to the jndi_initial_context_factory property of this JavaMessageServiceConnection.
        :type jndi_initial_context_factory: str

        :param jndi_security_principal:
            The value to assign to the jndi_security_principal property of this JavaMessageServiceConnection.
        :type jndi_security_principal: str

        :param connection_url:
            The value to assign to the connection_url property of this JavaMessageServiceConnection.
        :type connection_url: str

        :param connection_factory:
            The value to assign to the connection_factory property of this JavaMessageServiceConnection.
        :type connection_factory: str

        :param security_protocol:
            The value to assign to the security_protocol property of this JavaMessageServiceConnection.
            Allowed values for this property are: "PLAIN", "TLS", "MTLS", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type security_protocol: str

        :param authentication_type:
            The value to assign to the authentication_type property of this JavaMessageServiceConnection.
            Allowed values for this property are: "NONE", "BASIC", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type authentication_type: str

        :param username:
            The value to assign to the username property of this JavaMessageServiceConnection.
        :type username: str

        :param private_ip:
            The value to assign to the private_ip property of this JavaMessageServiceConnection.
        :type private_ip: str

        :param jndi_security_credentials_secret_id:
            The value to assign to the jndi_security_credentials_secret_id property of this JavaMessageServiceConnection.
        :type jndi_security_credentials_secret_id: str

        :param password_secret_id:
            The value to assign to the password_secret_id property of this JavaMessageServiceConnection.
        :type password_secret_id: str

        :param trust_store_secret_id:
            The value to assign to the trust_store_secret_id property of this JavaMessageServiceConnection.
        :type trust_store_secret_id: str

        :param trust_store_password_secret_id:
            The value to assign to the trust_store_password_secret_id property of this JavaMessageServiceConnection.
        :type trust_store_password_secret_id: str

        :param key_store_secret_id:
            The value to assign to the key_store_secret_id property of this JavaMessageServiceConnection.
        :type key_store_secret_id: str

        :param key_store_password_secret_id:
            The value to assign to the key_store_password_secret_id property of this JavaMessageServiceConnection.
        :type key_store_password_secret_id: str

        :param ssl_key_password_secret_id:
            The value to assign to the ssl_key_password_secret_id property of this JavaMessageServiceConnection.
        :type ssl_key_password_secret_id: str

        """
        self.swagger_types = {
            'connection_type': 'str',
            'id': 'str',
            'display_name': 'str',
            'description': 'str',
            'compartment_id': 'str',
            'freeform_tags': 'dict(str, str)',
            'defined_tags': 'dict(str, dict(str, object))',
            'system_tags': 'dict(str, dict(str, object))',
            'lifecycle_state': 'str',
            'lifecycle_details': 'str',
            'time_created': 'datetime',
            'time_updated': 'datetime',
            'locks': 'list[ResourceLock]',
            'vault_id': 'str',
            'key_id': 'str',
            'ingress_ips': 'list[IngressIpDetails]',
            'nsg_ids': 'list[str]',
            'subnet_id': 'str',
            'routing_method': 'str',
            'does_use_secret_ids': 'bool',
            'technology_type': 'str',
            'should_use_jndi': 'bool',
            'jndi_connection_factory': 'str',
            'jndi_provider_url': 'str',
            'jndi_initial_context_factory': 'str',
            'jndi_security_principal': 'str',
            'connection_url': 'str',
            'connection_factory': 'str',
            'security_protocol': 'str',
            'authentication_type': 'str',
            'username': 'str',
            'private_ip': 'str',
            'jndi_security_credentials_secret_id': 'str',
            'password_secret_id': 'str',
            'trust_store_secret_id': 'str',
            'trust_store_password_secret_id': 'str',
            'key_store_secret_id': 'str',
            'key_store_password_secret_id': 'str',
            'ssl_key_password_secret_id': 'str'
        }

        self.attribute_map = {
            'connection_type': 'connectionType',
            'id': 'id',
            'display_name': 'displayName',
            'description': 'description',
            'compartment_id': 'compartmentId',
            'freeform_tags': 'freeformTags',
            'defined_tags': 'definedTags',
            'system_tags': 'systemTags',
            'lifecycle_state': 'lifecycleState',
            'lifecycle_details': 'lifecycleDetails',
            'time_created': 'timeCreated',
            'time_updated': 'timeUpdated',
            'locks': 'locks',
            'vault_id': 'vaultId',
            'key_id': 'keyId',
            'ingress_ips': 'ingressIps',
            'nsg_ids': 'nsgIds',
            'subnet_id': 'subnetId',
            'routing_method': 'routingMethod',
            'does_use_secret_ids': 'doesUseSecretIds',
            'technology_type': 'technologyType',
            'should_use_jndi': 'shouldUseJndi',
            'jndi_connection_factory': 'jndiConnectionFactory',
            'jndi_provider_url': 'jndiProviderUrl',
            'jndi_initial_context_factory': 'jndiInitialContextFactory',
            'jndi_security_principal': 'jndiSecurityPrincipal',
            'connection_url': 'connectionUrl',
            'connection_factory': 'connectionFactory',
            'security_protocol': 'securityProtocol',
            'authentication_type': 'authenticationType',
            'username': 'username',
            'private_ip': 'privateIp',
            'jndi_security_credentials_secret_id': 'jndiSecurityCredentialsSecretId',
            'password_secret_id': 'passwordSecretId',
            'trust_store_secret_id': 'trustStoreSecretId',
            'trust_store_password_secret_id': 'trustStorePasswordSecretId',
            'key_store_secret_id': 'keyStoreSecretId',
            'key_store_password_secret_id': 'keyStorePasswordSecretId',
            'ssl_key_password_secret_id': 'sslKeyPasswordSecretId'
        }

        self._connection_type = None
        self._id = None
        self._display_name = None
        self._description = None
        self._compartment_id = None
        self._freeform_tags = None
        self._defined_tags = None
        self._system_tags = None
        self._lifecycle_state = None
        self._lifecycle_details = None
        self._time_created = None
        self._time_updated = None
        self._locks = None
        self._vault_id = None
        self._key_id = None
        self._ingress_ips = None
        self._nsg_ids = None
        self._subnet_id = None
        self._routing_method = None
        self._does_use_secret_ids = None
        self._technology_type = None
        self._should_use_jndi = None
        self._jndi_connection_factory = None
        self._jndi_provider_url = None
        self._jndi_initial_context_factory = None
        self._jndi_security_principal = None
        self._connection_url = None
        self._connection_factory = None
        self._security_protocol = None
        self._authentication_type = None
        self._username = None
        self._private_ip = None
        self._jndi_security_credentials_secret_id = None
        self._password_secret_id = None
        self._trust_store_secret_id = None
        self._trust_store_password_secret_id = None
        self._key_store_secret_id = None
        self._key_store_password_secret_id = None
        self._ssl_key_password_secret_id = None
        self._connection_type = 'JAVA_MESSAGE_SERVICE'

    @property
    def technology_type(self):
        """
        **[Required]** Gets the technology_type of this JavaMessageServiceConnection.
        The Java Message Service technology type.

        Allowed values for this property are: "ORACLE_WEBLOGIC_JMS", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The technology_type of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._technology_type

    @technology_type.setter
    def technology_type(self, technology_type):
        """
        Sets the technology_type of this JavaMessageServiceConnection.
        The Java Message Service technology type.


        :param technology_type: The technology_type of this JavaMessageServiceConnection.
        :type: str
        """
        allowed_values = ["ORACLE_WEBLOGIC_JMS"]
        if not value_allowed_none_or_none_sentinel(technology_type, allowed_values):
            technology_type = 'UNKNOWN_ENUM_VALUE'
        self._technology_type = technology_type

    @property
    def should_use_jndi(self):
        """
        **[Required]** Gets the should_use_jndi of this JavaMessageServiceConnection.
        If set to true, Java Naming and Directory Interface (JNDI) properties should be provided.


        :return: The should_use_jndi of this JavaMessageServiceConnection.
        :rtype: bool
        """
        return self._should_use_jndi

    @should_use_jndi.setter
    def should_use_jndi(self, should_use_jndi):
        """
        Sets the should_use_jndi of this JavaMessageServiceConnection.
        If set to true, Java Naming and Directory Interface (JNDI) properties should be provided.


        :param should_use_jndi: The should_use_jndi of this JavaMessageServiceConnection.
        :type: bool
        """
        self._should_use_jndi = should_use_jndi

    @property
    def jndi_connection_factory(self):
        """
        Gets the jndi_connection_factory of this JavaMessageServiceConnection.
        The Connection Factory can be looked up using this name.
        e.g.: 'ConnectionFactory'


        :return: The jndi_connection_factory of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._jndi_connection_factory

    @jndi_connection_factory.setter
    def jndi_connection_factory(self, jndi_connection_factory):
        """
        Sets the jndi_connection_factory of this JavaMessageServiceConnection.
        The Connection Factory can be looked up using this name.
        e.g.: 'ConnectionFactory'


        :param jndi_connection_factory: The jndi_connection_factory of this JavaMessageServiceConnection.
        :type: str
        """
        self._jndi_connection_factory = jndi_connection_factory

    @property
    def jndi_provider_url(self):
        """
        Gets the jndi_provider_url of this JavaMessageServiceConnection.
        The URL that Java Message Service will use to contact the JNDI provider.
        e.g.: 'tcp://myjms.host.domain:61616?jms.prefetchPolicy.all=1000'


        :return: The jndi_provider_url of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._jndi_provider_url

    @jndi_provider_url.setter
    def jndi_provider_url(self, jndi_provider_url):
        """
        Sets the jndi_provider_url of this JavaMessageServiceConnection.
        The URL that Java Message Service will use to contact the JNDI provider.
        e.g.: 'tcp://myjms.host.domain:61616?jms.prefetchPolicy.all=1000'


        :param jndi_provider_url: The jndi_provider_url of this JavaMessageServiceConnection.
        :type: str
        """
        self._jndi_provider_url = jndi_provider_url

    @property
    def jndi_initial_context_factory(self):
        """
        Gets the jndi_initial_context_factory of this JavaMessageServiceConnection.
        The implementation of javax.naming.spi.InitialContextFactory interface
        that the client uses to obtain initial naming context.
        e.g.: 'org.apache.activemq.jndi.ActiveMQInitialContextFactory'


        :return: The jndi_initial_context_factory of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._jndi_initial_context_factory

    @jndi_initial_context_factory.setter
    def jndi_initial_context_factory(self, jndi_initial_context_factory):
        """
        Sets the jndi_initial_context_factory of this JavaMessageServiceConnection.
        The implementation of javax.naming.spi.InitialContextFactory interface
        that the client uses to obtain initial naming context.
        e.g.: 'org.apache.activemq.jndi.ActiveMQInitialContextFactory'


        :param jndi_initial_context_factory: The jndi_initial_context_factory of this JavaMessageServiceConnection.
        :type: str
        """
        self._jndi_initial_context_factory = jndi_initial_context_factory

    @property
    def jndi_security_principal(self):
        """
        Gets the jndi_security_principal of this JavaMessageServiceConnection.
        Specifies the identity of the principal (user) to be authenticated.
        e.g.: 'admin2'


        :return: The jndi_security_principal of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._jndi_security_principal

    @jndi_security_principal.setter
    def jndi_security_principal(self, jndi_security_principal):
        """
        Sets the jndi_security_principal of this JavaMessageServiceConnection.
        Specifies the identity of the principal (user) to be authenticated.
        e.g.: 'admin2'


        :param jndi_security_principal: The jndi_security_principal of this JavaMessageServiceConnection.
        :type: str
        """
        self._jndi_security_principal = jndi_security_principal

    @property
    def connection_url(self):
        """
        Gets the connection_url of this JavaMessageServiceConnection.
        Connectin URL of the Java Message Service, specifying the protocol, host, and port.
        e.g.: 'mq://myjms.host.domain:7676'


        :return: The connection_url of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._connection_url

    @connection_url.setter
    def connection_url(self, connection_url):
        """
        Sets the connection_url of this JavaMessageServiceConnection.
        Connectin URL of the Java Message Service, specifying the protocol, host, and port.
        e.g.: 'mq://myjms.host.domain:7676'


        :param connection_url: The connection_url of this JavaMessageServiceConnection.
        :type: str
        """
        self._connection_url = connection_url

    @property
    def connection_factory(self):
        """
        Gets the connection_factory of this JavaMessageServiceConnection.
        The of Java class implementing javax.jms.ConnectionFactory interface
        supplied by the Java Message Service provider.
        e.g.: 'com.stc.jmsjca.core.JConnectionFactoryXA'


        :return: The connection_factory of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._connection_factory

    @connection_factory.setter
    def connection_factory(self, connection_factory):
        """
        Sets the connection_factory of this JavaMessageServiceConnection.
        The of Java class implementing javax.jms.ConnectionFactory interface
        supplied by the Java Message Service provider.
        e.g.: 'com.stc.jmsjca.core.JConnectionFactoryXA'


        :param connection_factory: The connection_factory of this JavaMessageServiceConnection.
        :type: str
        """
        self._connection_factory = connection_factory

    @property
    def security_protocol(self):
        """
        Gets the security_protocol of this JavaMessageServiceConnection.
        Security protocol for Java Message Service. If not provided, default is PLAIN.
        Optional until 2024-06-27, in the release after it will be made required.

        Allowed values for this property are: "PLAIN", "TLS", "MTLS", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The security_protocol of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._security_protocol

    @security_protocol.setter
    def security_protocol(self, security_protocol):
        """
        Sets the security_protocol of this JavaMessageServiceConnection.
        Security protocol for Java Message Service. If not provided, default is PLAIN.
        Optional until 2024-06-27, in the release after it will be made required.


        :param security_protocol: The security_protocol of this JavaMessageServiceConnection.
        :type: str
        """
        allowed_values = ["PLAIN", "TLS", "MTLS"]
        if not value_allowed_none_or_none_sentinel(security_protocol, allowed_values):
            security_protocol = 'UNKNOWN_ENUM_VALUE'
        self._security_protocol = security_protocol

    @property
    def authentication_type(self):
        """
        Gets the authentication_type of this JavaMessageServiceConnection.
        Authentication type for Java Message Service.  If not provided, default is NONE.
        Optional until 2024-06-27, in the release after it will be made required.

        Allowed values for this property are: "NONE", "BASIC", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The authentication_type of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._authentication_type

    @authentication_type.setter
    def authentication_type(self, authentication_type):
        """
        Sets the authentication_type of this JavaMessageServiceConnection.
        Authentication type for Java Message Service.  If not provided, default is NONE.
        Optional until 2024-06-27, in the release after it will be made required.


        :param authentication_type: The authentication_type of this JavaMessageServiceConnection.
        :type: str
        """
        allowed_values = ["NONE", "BASIC"]
        if not value_allowed_none_or_none_sentinel(authentication_type, allowed_values):
            authentication_type = 'UNKNOWN_ENUM_VALUE'
        self._authentication_type = authentication_type

    @property
    def username(self):
        """
        Gets the username of this JavaMessageServiceConnection.
        The username Oracle GoldenGate uses to connect to the Java Message Service.
        This username must already exist and be available by the Java Message Service to be connected to.


        :return: The username of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._username

    @username.setter
    def username(self, username):
        """
        Sets the username of this JavaMessageServiceConnection.
        The username Oracle GoldenGate uses to connect to the Java Message Service.
        This username must already exist and be available by the Java Message Service to be connected to.


        :param username: The username of this JavaMessageServiceConnection.
        :type: str
        """
        self._username = username

    @property
    def private_ip(self):
        """
        Gets the private_ip of this JavaMessageServiceConnection.
        Deprecated: this field will be removed in future versions. Either specify the private IP in the connectionString or host
        field, or make sure the host name is resolvable in the target VCN.

        The private IP address of the connection's endpoint in the customer's VCN, typically a
        database endpoint or a big data endpoint (e.g. Kafka bootstrap server).
        In case the privateIp is provided, the subnetId must also be provided.
        In case the privateIp (and the subnetId) is not provided it is assumed the datasource is publicly accessible.
        In case the connection is accessible only privately, the lack of privateIp will result in not being able to access the connection.


        :return: The private_ip of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._private_ip

    @private_ip.setter
    def private_ip(self, private_ip):
        """
        Sets the private_ip of this JavaMessageServiceConnection.
        Deprecated: this field will be removed in future versions. Either specify the private IP in the connectionString or host
        field, or make sure the host name is resolvable in the target VCN.

        The private IP address of the connection's endpoint in the customer's VCN, typically a
        database endpoint or a big data endpoint (e.g. Kafka bootstrap server).
        In case the privateIp is provided, the subnetId must also be provided.
        In case the privateIp (and the subnetId) is not provided it is assumed the datasource is publicly accessible.
        In case the connection is accessible only privately, the lack of privateIp will result in not being able to access the connection.


        :param private_ip: The private_ip of this JavaMessageServiceConnection.
        :type: str
        """
        self._private_ip = private_ip

    @property
    def jndi_security_credentials_secret_id(self):
        """
        Gets the jndi_security_credentials_secret_id of this JavaMessageServiceConnection.
        The `OCID`__ of the Secret where the security credentials are stored associated to the principal.
        Note: When provided, 'jndiSecurityCredentials' field must not be provided.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The jndi_security_credentials_secret_id of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._jndi_security_credentials_secret_id

    @jndi_security_credentials_secret_id.setter
    def jndi_security_credentials_secret_id(self, jndi_security_credentials_secret_id):
        """
        Sets the jndi_security_credentials_secret_id of this JavaMessageServiceConnection.
        The `OCID`__ of the Secret where the security credentials are stored associated to the principal.
        Note: When provided, 'jndiSecurityCredentials' field must not be provided.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param jndi_security_credentials_secret_id: The jndi_security_credentials_secret_id of this JavaMessageServiceConnection.
        :type: str
        """
        self._jndi_security_credentials_secret_id = jndi_security_credentials_secret_id

    @property
    def password_secret_id(self):
        """
        Gets the password_secret_id of this JavaMessageServiceConnection.
        The `OCID`__ of the Secret where the password is stored,
        that Oracle GoldenGate uses to connect the associated Java Message Service.
        Note: When provided, 'password' field must not be provided.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The password_secret_id of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._password_secret_id

    @password_secret_id.setter
    def password_secret_id(self, password_secret_id):
        """
        Sets the password_secret_id of this JavaMessageServiceConnection.
        The `OCID`__ of the Secret where the password is stored,
        that Oracle GoldenGate uses to connect the associated Java Message Service.
        Note: When provided, 'password' field must not be provided.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param password_secret_id: The password_secret_id of this JavaMessageServiceConnection.
        :type: str
        """
        self._password_secret_id = password_secret_id

    @property
    def trust_store_secret_id(self):
        """
        Gets the trust_store_secret_id of this JavaMessageServiceConnection.
        The `OCID`__ of the Secret where the content of the TrustStore file is stored.
        Note: When provided, 'trustStore' field must not be provided.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The trust_store_secret_id of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._trust_store_secret_id

    @trust_store_secret_id.setter
    def trust_store_secret_id(self, trust_store_secret_id):
        """
        Sets the trust_store_secret_id of this JavaMessageServiceConnection.
        The `OCID`__ of the Secret where the content of the TrustStore file is stored.
        Note: When provided, 'trustStore' field must not be provided.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param trust_store_secret_id: The trust_store_secret_id of this JavaMessageServiceConnection.
        :type: str
        """
        self._trust_store_secret_id = trust_store_secret_id

    @property
    def trust_store_password_secret_id(self):
        """
        Gets the trust_store_password_secret_id of this JavaMessageServiceConnection.
        The `OCID`__ of the Secret where the TrustStore password is stored.
        Note: When provided, 'trustStorePassword' field must not be provided.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The trust_store_password_secret_id of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._trust_store_password_secret_id

    @trust_store_password_secret_id.setter
    def trust_store_password_secret_id(self, trust_store_password_secret_id):
        """
        Sets the trust_store_password_secret_id of this JavaMessageServiceConnection.
        The `OCID`__ of the Secret where the TrustStore password is stored.
        Note: When provided, 'trustStorePassword' field must not be provided.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param trust_store_password_secret_id: The trust_store_password_secret_id of this JavaMessageServiceConnection.
        :type: str
        """
        self._trust_store_password_secret_id = trust_store_password_secret_id

    @property
    def key_store_secret_id(self):
        """
        Gets the key_store_secret_id of this JavaMessageServiceConnection.
        The `OCID`__ of the Secret where the content of the KeyStore file is stored.
        Note: When provided, 'keyStore' field must not be provided.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The key_store_secret_id of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._key_store_secret_id

    @key_store_secret_id.setter
    def key_store_secret_id(self, key_store_secret_id):
        """
        Sets the key_store_secret_id of this JavaMessageServiceConnection.
        The `OCID`__ of the Secret where the content of the KeyStore file is stored.
        Note: When provided, 'keyStore' field must not be provided.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param key_store_secret_id: The key_store_secret_id of this JavaMessageServiceConnection.
        :type: str
        """
        self._key_store_secret_id = key_store_secret_id

    @property
    def key_store_password_secret_id(self):
        """
        Gets the key_store_password_secret_id of this JavaMessageServiceConnection.
        The `OCID`__ of the Secret where the KeyStore password is stored.
        Note: When provided, 'keyStorePassword' field must not be provided.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The key_store_password_secret_id of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._key_store_password_secret_id

    @key_store_password_secret_id.setter
    def key_store_password_secret_id(self, key_store_password_secret_id):
        """
        Sets the key_store_password_secret_id of this JavaMessageServiceConnection.
        The `OCID`__ of the Secret where the KeyStore password is stored.
        Note: When provided, 'keyStorePassword' field must not be provided.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param key_store_password_secret_id: The key_store_password_secret_id of this JavaMessageServiceConnection.
        :type: str
        """
        self._key_store_password_secret_id = key_store_password_secret_id

    @property
    def ssl_key_password_secret_id(self):
        """
        Gets the ssl_key_password_secret_id of this JavaMessageServiceConnection.
        The `OCID`__ of the Secret where the password is stored for the cert inside of the Keystore.
        In case it differs from the KeyStore password, it should be provided.
        Note: When provided, 'sslKeyPassword' field must not be provided.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The ssl_key_password_secret_id of this JavaMessageServiceConnection.
        :rtype: str
        """
        return self._ssl_key_password_secret_id

    @ssl_key_password_secret_id.setter
    def ssl_key_password_secret_id(self, ssl_key_password_secret_id):
        """
        Sets the ssl_key_password_secret_id of this JavaMessageServiceConnection.
        The `OCID`__ of the Secret where the password is stored for the cert inside of the Keystore.
        In case it differs from the KeyStore password, it should be provided.
        Note: When provided, 'sslKeyPassword' field must not be provided.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param ssl_key_password_secret_id: The ssl_key_password_secret_id of this JavaMessageServiceConnection.
        :type: str
        """
        self._ssl_key_password_secret_id = ssl_key_password_secret_id

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other
